var Superloop = (function() {
    var superloop = {},
        bridgeReady = false,
        bridgeRegisterHandlers = [],
        bridgeCallHandlers = [];

    function init() {
        // 初始化WebViewJavascriptBridge
        superloop.bridge.__init__();
    };
    ! function extension() {
        $.extend($.fn, {
            // 通过URL加载图片
            loadImage: function(url) {
                var _this = this[0],
                    imgObj = new Image();
                imgObj.onload = function() {
                    _this.src = url;
                    imgObj = null;
                };
                imgObj.onerror = function() {
                    imgObj = null;
                };
                imgObj.src = url;
            },
            tap: function(callback, stopPropagation) {
                var touch = false,
                    cancelBubble = typeof stopPropagation === 'boolean' ? stopPropagation : true;
                this.on('touchstart', function() {
                    touch = true;
                });
                this.on('touchmove', function() {
                    touch = false;
                });
                this.on('touchend', function(e) {
                    if (touch) {
                        touch = false;
                        callback.call(this, e);
                        if (cancelBubble) {
                            // 不冒泡
                            e.stopPropagation();
                            e.preventDefault();
                        }
                    }
                });
                return this;
            },
            unactive: function() {
                if (this.length > 1) {
                    this.each(function(_, element) {
                        $(element).unactive();
                    });
                    return true;
                }
                var $this = $(this);
                $this.off();
            },
            active: function(propertys, callback, stopPropagation) {
                if (this.length > 1) {
                    this.each(function(_, element) {
                        $(element).active(propertys, callback);
                    });
                    return true;
                }
                var cancelBubble = typeof stopPropagation === 'boolean' ? stopPropagation : true;
                if (propertys) {
                    typeof propertys === 'boolean' && (cancelBubble = callback);
                    typeof callback === 'boolean' && (cancelBubble = callback);
                    $.isFunction(propertys) && (callback = propertys);
                }

                function fn() {}
                var $this = $(this),
                    settings = {
                        default: {},
                        active: {}
                    },
                    styles = {},
                    touch = false,
                    longTapTimeout;

                function cancelLongTap() {
                    longTapTimeout && clearTimeout(longTapTimeout);
                    longTapTimeout = null;
                }
                propertys = $.isPlainObject(propertys) && propertys || settings;
                callback = callback || fn;
                settings.default.background = propertys.default.background || '#fff';
                settings.active.background = propertys.active.background || '#c0c7d1';
                // 背景色
                styles.default = {
                    background: settings.default.background
                };
                styles.active = {
                    background: settings.active.background
                };
                // 内容颜色
                propertys.default.color && (styles.default.color = propertys.default.color);
                propertys.active.color && (styles.active.color = propertys.active.color);
                $this.on('touchstart', function() {
                    longTapTimeout = setTimeout(function() {
                        touch = false;
                        $this.css(styles.active);
                    }, 300);
                    touch = true;
                });
                $this.on('touchmove', function() {
                    touch && (touch = false);
                    cancelLongTap();
                });
                $this.on('touchend', function(e) {
                    cancelLongTap();
                    setTimeout(function() {
                        $this.css(styles.default);
                    }, 100);
                    if (touch) {
                        $this.css(styles.active);
                        touch = false;
                        callback.call(this, e);
                        if (cancelBubble) {
                            // 不冒泡
                            e.stopPropagation();
                            e.preventDefault();
                        }
                    }
                });
                $this.on('touchcancel', function() {
                    touch && (touch = false);
                    setTimeout(function() {
                        $this.css(styles.default);
                    }, 100);
                });
                return this;
            }
        });
        $.isBlank = function(obj) {
            //检测是否为空
            return (!obj || $.trim(obj) === "");
        };
    }();

    // http请求方法
    superloop.httpRequest = function(options) {
        //superloop.AJAX(options);
        superloop.bridge.httpRequest(options);
    };
    // AJAX请求
    superloop.AJAX = function(options) {
        $.ajax({
            url: options.url,
            type: options.type,
            dataType: 'json',
            data: options.data,
            headers: {
                'Authorization': Superloop.User.Authorization
            },
            success: function(data) {
                options.success(data);
            },
            error: function() {
                options.error();
            }
        });
    };

    superloop.URL = {
        API: {
            domain: 'https://api.superloop.com.cn'
        },
        Image: {
            domain: 'http://img.superloop.com.cn/'
        }
    };

    superloop.User = {
        id: -1,
        Authorization: ''
    };

    //判断返回数据是否含有特殊字符
    superloop.alert = function(options) {
        superloop.bridge.callHandler('alert', { content: options.content });
    };
    //他人页消息
    superloop.IM = function(data) {
        superloop.bridge.callHandler('IM', data);
    };
    //他人打电话
    superloop.call = function(data) {
        superloop.bridge.callHandler('call', data);
    };
    //他人页的评价
    superloop.getEvaluate = function(data) {
        superloop.bridge.callHandler('getEvaluate', data);
    };
    //他人的话题
    superloop.getTopics = function(data) {
        superloop.bridge.callHandler('getTopics', data);
    };
    //查看他人头像
    superloop.imgView = function(data) {
        superloop.bridge.callHandler('imgView', data);
    };
    //查看他人粉丝
    superloop.getOtherFans = function(data) {
        superloop.bridge.callHandler('getOtherFans', data);
    };
    //查看他人关注
    superloop.getOtherFollow = function(data) {
        superloop.bridge.callHandler('getOtherFollow', data);
    };
    //关注和取消关注之后刷新
    superloop.updateContacts = function() {
        superloop.bridge.callHandler('updateContacts');
    };
    //设置头信息
    superloop.setUser = function(args) {
        if(!args) return false;
        var data = {
            nickname: args.nickname || '',
            avatar: args.avatar || '',
            bio: args.bio || ''
        };
        superloop.bridge.callHandler('setUser', data);
    };
    //请求接口完成
    superloop.onDataFinished = function() {
        superloop.bridge.callHandler('onDataFinished');
    };
    //分享自己与朋友
    superloop.share = function(){
        superloop.bridge.callHandler('share');
    };
    superloop.loading = {
        show: function(options) {
            //alert('show loading');
            var data = { show: 1 };
            if (options && options.content) {
                data.content = options.content;
            }
            superloop.bridge.callHandler('loading', data);
        },
        close: function(options) {
            //alert('close loading');
            var data = { show: 0 };
            if (options && options.content) {
                data.content = options.content;
            }
            superloop.bridge.callHandler('loading', data);
        }
    };
    superloop.Reload = function(callback) {
        superloop.bridge.registerHandler('reload', function(data, response) {
            callback()
        });
    };
    // WebViewJavaScriptBridge
    superloop.bridge = {
        __init__: function() {
            // BridgeReady
            if (bridgeReady) return false;
            bridgeReady = true;
            var _this = this;
            document.addEventListener('WebViewJavascriptBridgeReady', function() {
                if (!_this.isAvailable()) return false;
                var temp;
                while (temp = bridgeRegisterHandlers.shift()) {
                    WebViewJavascriptBridge.registerHandler(temp.handlerName, temp.handler);
                }
                while (temp = bridgeCallHandlers.shift()) {
                    WebViewJavascriptBridge.callHandler(temp.handlerName, temp.data, temp.callback);
                }
            }, false);
        },

        // 是否可使用WebViewJavascriptBridge
        isAvailable: function() {
            return typeof(WebViewJavascriptBridge) !== 'undefined';
        },
        // 绑定提供给原生调用的js方法
        registerHandler: function(handlerName, handler) {
            if (this.isAvailable()) {
                WebViewJavascriptBridge.registerHandler(handlerName, handler);
            } else {
                bridgeRegisterHandlers.push({ handlerName: handlerName, handler: handler });
            }
        },
        // 调用原生提供的方法
        callHandler: function(handlerName, data, responseCallback) {
            var callback = function(response) {
                responseCallback($.parseJSON(response));
            };
            if (this.isAvailable()) {
                WebViewJavascriptBridge.callHandler(handlerName, data, callback);
            } else {
                bridgeCallHandlers.push({ handlerName: handlerName, data: data, callback: callback });
            }
        },
        // 页面跳转
        navigate: function(page, url, target, args) {
            target = target == undefined ? '_blank' : target;
            var data = { page: page, url: url, target: target, args: args };
            this.callHandler('navigate', data);
        },

        getUser: function(callback) {
            this.callHandler('getUser', null, function(response) {
                superloop.User = response;
                callback();
            });
        },
        // Native httpRequest
        httpRequest: function(options) {
            function fn() {}
            var settings = options || {};
            var args = {},
                dataType = settings.dataType || '',
                success = settings.success || fn,
                error = settings.error || fn;
            if ($.isBlank(settings.url)) {
                error(103, '缺少必要参数');
                return false;
            }
            args.type = settings.type || 'GET'
            args.url = settings.url;
            args.headers = settings.headers;
            args.params = settings.params; // GET请求参数
            args.data = settings.data; // POST请求数据体
            this.callHandler('httpRequest', args, function(response) {
                if (response.code == 601 &&
                    !$.isBlank(response.responseText)) {
                    var data = response.responseText;
                    if (dataType === 'json') {
                        try {
                            data = $.parseJSON(data);
                        } catch (e) {

                        }
                    }
                    success(data);
                } else if (response.code) {
                    error(response.code, response.message);
                } else {
                    // 未知错误
                    error(404, '未知错误');
                }
            });
        },
        close: function() {
            this.callHandler('close');
        }
    };
    superloop.navigate = {
        // 关注的话题
        focusTopic: function() {
            Superloop.bridge.navigate('FocusTopic');
        },
        // 我的评论
        mineComment: function() {
            Superloop.bridge.navigate('MineComment');
        },
        // 我的话题
        mineTopic: function() {
            Superloop.bridge.navigate('MineTopic');
        },
        // 联系人列表
        linkman: function() {
            Superloop.bridge.navigate('LinkMan');
        },
        // 粉丝列表
        fans: function() {
            Superloop.bridge.navigate('UserFans');
        },
        // 我的收藏
        collect: function() {
            Superloop.bridge.navigate('MineFavorite');
        },
        // 通话资费
        callCharges: function() {
            Superloop.bridge.navigate('CallCharges');
        },
        // 账户余额
        wallet: function(args) {
            Superloop.bridge.navigate('Wallet', '', '_blank', args);
        },
        // 认证审核
        authentication: function() {
            Superloop.bridge.navigate('Authentication');
        },
        //个人资料详情页
        userDetail: function() {
            Superloop.bridge.navigate('MineDetail', 'user_detail.html');
        },
        //基本信息
        userInformation: function() {
            Superloop.bridge.navigate('MineBasicInfo');
        },
        //修改简介
        abstractModify: function() {
            Superloop.bridge.navigate('AbstractModify');
        },
        //修改标签
        labelModify: function(args) {
            Superloop.bridge.navigate('LabelModify', '', '_blank', args);
        },
        //注册协议
        agreement: function() {
            Superloop.bridge.navigate('', 'agreement.html');
        },
        //别人对我的评价
        avatorStar: function(args) {
            Superloop.bridge.navigate('Appraise', '', '_blank', args);
        },
        //消息免骚扰
        harassment: function() {
            Superloop.bridge.navigate('Harassment');
        }
    }
    init();
    return superloop;
})();

window.Superloop = Superloop;
